@@ -21,9 +21,11 @@ from cprocsp import csp, cryptoapi
 from . import KeysetException
 
 if sys.version_info >= (3,):
-        ord = lambda x: x
-        unicode = lambda x: str(x, sys.getdefaultencoding())
-
+    ord = lambda x: x
+    _decode = lambda x: str(x, sys.getdefaultencoding())
+    unicode = str
+else:
+    _decode = unicode
 
 def _rarsplit(fn):
     res = re.split(r'[\\/]', fn, maxsplit=1)
@@ -44,9 +46,9 @@ def _cert_info(certdata):
     tc = csp.Cert(certdata)
     ci = csp.CertInfo(tc)
     return dict(
-        name=unicode(ci.name()),
-        thumb=unicode(hexlify(tc.thumbprint())),
-        issuer=unicode(ci.issuer()),
+        name=_decode(ci.name()),
+        thumb=_decode(hexlify(tc.thumbprint())),
+        issuer=_decode(ci.issuer()),
         serial=':'.join(hex(ord(x))[2:] for x in
                         reversed(ci.serial())),
     )
@@ -85,18 +87,27 @@ class hdimage(object):
         :user: container owner, defaults to current user
         """
 
-        self.cont = str(r'\\.\HDIMAGE\{0}'.format(name))
+        self.cont = b'\\\\.\\HDIMAGE\\' + cryptoapi._from_hex(name)
         self.name = name
-        self.section = 'KeyDevices\\passwords\\"{0}"'.format(self.name)
         self.user = user or getpass.getuser()
         self.certs = {}
 
         self.configfile = '/var/opt/cprocsp/users/{0}/local.ini'.format(self.user)
-        self.section = 'KeyDevices\\passwords\\"{0}"'.format(self.name)
+        self.section = r'KeyDevices\passwords\"{0}"'.format(self.name)
 
         self.broken = False
+        self.empty = True
         try:
             self.ctx = csp.Crypt(self.cont, csp.PROV_GOST_2001_DH, csp.CRYPT_SILENT)
+            self.empty = False
+        except ValueError:
+            try:
+                self.ctx = csp.Crypt(
+                    self.cont, csp.PROV_GOST_2001_DH, csp.CRYPT_NEWKEYSET | csp.CRYPT_SILENT)
+            except:
+                self.broken = True
+                self.ctx = None
+        else:
             key = self.ctx.get_key()
             try:
                 cert = key.extract_cert(csp.AT_KEYEXCHANGE)
@@ -108,21 +119,21 @@ class hdimage(object):
                 self.certs['AT_SIGNATURE'] = _cert_info(cert)
             except:
                 pass
-        except ValueError:
-            try:
-                self.ctx = csp.Crypt(
-                    self.cont, csp.PROV_GOST_2001_DH, csp.CRYPT_NEWKEYSET | csp.CRYPT_SILENT)
-            except:
-                self.broken = True
-                self.ctx = None
 
         if not self.broken:
             self.shortcut = self.ctx.uniq_name()
         else:
-            self.shortcut = eval(self.config.get(self.section, 'shortcut'))
-        self.folder = self.shortcut.split('\\')[2]
-        self.dirname = '/var/opt/cprocsp/keys/{0}/{1}'.format(self.user, self.folder)
-        self.empty = not os.path.exists(self.dirname)
+            if self.config.has_section(self.section):
+                self.shortcut = eval(self.config.get(self.section, 'shortcut'))
+            else:
+                self.shortcut = None
+        if self.shortcut:
+            self.folder = self.shortcut.split('\\')[2]
+            self.dirname = '/var/opt/cprocsp/keys/{0}/{1}'.format(self.user, self.folder)
+            self.empty = not os.path.exists(self.dirname)
+        else:
+            self.folder = self.dirname = None
+            self.empty = True
 
     def initialize(self):
         """
@@ -130,9 +141,6 @@ class hdimage(object):
 
         :returns: None
         """
-        if not self.empty:
-            return
-
         if not os.path.isdir(self.dirname):
             os.makedirs(self.dirname)
 
@@ -148,9 +156,11 @@ class hdimage(object):
 
         :filename: archive file name
         :arccls: archive management class, compatible with zipfile.ZipFile
-        :returns: None
+        :returns: saved filename
         """
 
+        if not filename:
+            filename = self.name.replace('\\', '_') + '.zip'
         with ZipFile(filename, mode='w') as zf:
             if self.ctx and self.dirname:
                 for fn in iglob(os.path.join(self.dirname, '*.key')):
@@ -165,6 +175,7 @@ class hdimage(object):
                     cert = key.extract_cert()
                     zf.writestr('{0}.cer'.format(keyname), cert)
                 zf.close()
+        return filename
 
     def delete(self):
         """
@@ -209,7 +220,7 @@ class hdimage(object):
                     non_standard=non_standard_certs)
 
     @classmethod
-    def restore(cls, filename, filetype=None, paswd=None):
+    def restore(cls, filename, filetype=None, paswd=None, force=False):
         """Create and load container from zip archive contents
 
         :filename: archive file name
@@ -230,9 +241,16 @@ class hdimage(object):
         names = zf.namelist()
         struct = cls.check_structure(names)
         value = zf.read('{0}/name.key'.format(struct['dirname']))
-        name = decoder.decode(value)[0][0]
+        real_name = bytes(decoder.decode(value)[0][0])
+        name = cryptoapi._to_hex(real_name)
         res = cls(name)
-        _ensure(res.empty, 'Container {0} exists'.format(name))
+        _ensure(res.empty or force, "Container '{0}' exists".format(name))
+
+        if not res.empty or force:
+            res.delete()
+            del res
+        res = cls(name)
+        _ensure(res.empty, "Couldn't delete container '{0}'".format(name))
         res.initialize()
         for fn in struct['files']:
             data = zf.read('{0}/{1}'.format(struct['dirname'], fn))
